#version 430 core

// !IMPORTANT!
// If the locations are changed, they need to be changed in the program aswell.

layout(location = 0) in vec3 coordinates;
layout(location = 1) in vec3 normalVec;
layout(location = 9) in vec2 UVs;
layout(location = 11) in ivec4 boneIds;
layout(location = 12) in vec4 boneWeights;

out vertexInformation
{
	vec3 vertexWorldPosition;
} vertexInf;
out vec3 normalWorldOrientation;
out vec2 fragUVs;
out vec2 fragLMUVs;

layout(location = 2) uniform mat4 modelMatrix;
layout(location = 3) uniform mat4 normalMatrix;
layout(location = 4) uniform mat4 projectedCameraMatrix;
layout(location = 13) uniform mat4 bones[40];

void main(void)
{
	mat4 bTransformation = bones[boneIds.x] * boneWeights.x;
	if (boneIds.y >= 0)	bTransformation += bones[boneIds.y] * boneWeights.y;
	if (boneIds.z >= 0)	bTransformation += bones[boneIds.z] * boneWeights.z;
	if (boneIds.w >= 0)	bTransformation += bones[boneIds.w] * boneWeights.w;

	vec4 bCoordinates = bTransformation * vec4(coordinates, 1.0);
	vec4 bNormalVec = bTransformation * vec4(normalVec, 1.0);
	
	normalWorldOrientation = vec3(normalMatrix * bNormalVec);
	vec3 worldPos = vec3(modelMatrix * bCoordinates);
	vertexInf.vertexWorldPosition = worldPos;
	fragUVs = UVs;
	fragLMUVs = vec2(1.0, 1.0);
	gl_Position = projectedCameraMatrix * vec4(worldPos, 1.0);
}