#version 430 core

// !IMPORTANT!
// If the locations are changed, they need to be changed in the program aswell.

in vertexInformation
{
	vec3 vertexWorldPosition;
} vertexInf;
in vec3 normalWorldOrientation;
in vec2 fragUVs;
in vec2 fragLMUVs;

layout(location = 5) uniform vec3 DLDirection;
layout(location = 6) uniform vec3 DLColor;
layout(location = 7) uniform vec3 cameraPos;
layout(location = 8) uniform vec4 materialAttr; //x=ambient, y=diffuse, z=specular, w=alpha
layout(location = 10) uniform float valuePercentage;
layout(location = 56) uniform bool useLightMap;

layout(location = 53) uniform sampler2D mainTexture;
layout(location = 54) uniform sampler2D lightMap;
layout(location = 55) uniform int celShadeLevel = 5;


const mat3 sobelX = mat3(
	 1.0,  2.0,  1.0,
	 0.0,  0.0,  0.0,
	-1.0, -2.0, -1.0
);
const mat3 sobelY = mat3(
	 1.0,  0.0, -1.0,
	 2.0,  0.0, -2.0,
	 1.0,  0.0, -1.0
);


vec3 phong(vec3 N, vec3 L, vec3 V, vec4 material, vec3 objColor, vec3 lightColor, bool useAtt, vec3 Att)
{
	float attentuation = 1.0;
	if(useAtt)
	{	
		float LightDistance = length(L);
		attentuation = 1/(Att.x + Att.y * LightDistance + Att.z * LightDistance * LightDistance);
	}
	L = normalize(L);
	vec3 R = reflect(-L, N);
	return (material.y * objColor * lightColor * max(0, dot(N, L)) + material.z * lightColor * pow(max(0, dot(R, V)), material.w)) * attentuation;
}

vec3 reduceValue(vec3 color, float percentage)
{
	// the conversions between the rgb and hsv color space are taken from https://stackoverflow.com/questions/15095909/from-rgb-to-hsv-in-opengl-glsl
	// convert into the hsv space 
	vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(color.bg, K.wz), vec4(color.gb, K.xy), step(color.b, color.g));
    vec4 q = mix(vec4(p.xyw, color.r), vec4(color.r, p.yzx), step(p.x, color.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    vec3 hsvColor =  vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
	
	// reduce the value
	hsvColor.y *= percentage;
	
	//convert back into rgb color
	K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
	vec3 r = abs(fract(hsvColor.xxx + K.xyz) * 6.0 - K.www);
	return hsvColor.z * mix(K.xxx, clamp(r - K.xxx, 0.0, 1.0), hsvColor.y);
}

vec3 celShading(vec3 color, vec3 light)
{
	float maxColor = max(max(light.x, light.y), light.z);
	float minColor = min(min(light.x, light.y), light.z);
	float lightness = (maxColor + minColor) * 0.5;

	lightness = round(lightness * celShadeLevel) / celShadeLevel;

	return color * lightness;
}

void main (void)
{
	vec3 N = normalize(normalWorldOrientation);
	vec3 V = normalize(cameraPos - vertexInf.vertexWorldPosition);
	
	vec3 texColor = vec3(texture(mainTexture, fragUVs));
	texColor = reduceValue(texColor, valuePercentage);
	vec3 color = texColor;
	vec3 light = vec3(1.0, 1.0, 1.0);

	if(!useLightMap)
	{
		light *= materialAttr.x;
		light += phong(N, normalize(-DLDirection), V, materialAttr, texColor, DLColor, false, vec3(1.0, 1.0, 1.0));
	}
	else	light *= vec3(texture(lightMap, fragLMUVs));
	
	color = celShading(color, light);

	gl_FragColor = vec4(color, 1.0);
}