#version 430

in vec2 fragTexels;
layout(location = 2) uniform sampler2D tex1;
layout(location = 3) uniform sampler2D tex2;

mat3 sx = mat3(1.0, 2.0, 1.0, 0.0, 0.0, 0.0, -1.0, -2.0, -1.0);
mat3 sy = mat3(1.0, 0.0, -1.0, 2.0, 0.0, -2.0, 1.0, 0.0, -1.0);

void main(void)
{
	vec3 diffuseColour = texelFetch(tex1, ivec2(fragTexels), 0).rgb;
	mat3 R;
	mat3 G;
	mat3 B;
	vec3 normalsSample;
	for(int i = 0; i < 3; i++)
	{
		for(int j = 0; j < 3; j++)
		{
			normalsSample = texelFetch(tex2, ivec2(fragTexels) + ivec2(i-1, j-1), 0).rgb;
			normalsSample = (normalsSample * 2) - vec3(1.0, 1.0, 1.0);
			R[i][j] = normalsSample.r;
			G[i][j] = normalsSample.g;
			B[i][j] = normalsSample.b;
		}
	}
	float RdX = dot(sx[0], R[0]) + dot(sx[1], R[1]) + dot(sx[2], R[2]);
	float RdY = dot(sy[0], R[0]) + dot(sy[1], R[1]) + dot(sy[2], R[2]);
	float GdX = dot(sx[0], G[0]) + dot(sx[1], G[1]) + dot(sx[2], G[2]);
	float GdY = dot(sy[0], G[0]) + dot(sy[1], G[1]) + dot(sy[2], G[2]);
	float BdX = dot(sx[0], B[0]) + dot(sx[1], B[1]) + dot(sx[2], B[2]);
	float BdY = dot(sy[0], B[0]) + dot(sy[1], B[1]) + dot(sy[2], B[2]);
	
	float JX = pow(RdX, 2.0) + pow(GdX, 2.0) + pow(BdX, 2.0);
	float JY = pow(RdY, 2.0) + pow(GdY, 2.0) + pow(BdY, 2.0);
	float JXY = RdX * RdY + GdX * GdY + BdX * BdY;
	
	float det = sqrt(abs(pow(JX, 2.0) - 2 * JX * JY + pow(JY, 2.0) + 4 * pow(JXY, 2.0)));
	float eigenValue = (JX + JY + det)/2.0;
	float mag = sqrt(eigenValue);
	if(mag < 0.75) mag = 0.0;
	
	//gl_FragColor = vec4(diffuseColour - vec3(grad), 1.0);
	//gl_FragColor = vec4((texelFetch(tex2, ivec2(fragTexels), 0).rgb * 2) - vec3(1.0, 1.0, 1.0), 1.0);
	gl_FragColor =  vec4(diffuseColour - vec3(mag), 1.0);
}
